package org.nuxeo.ecm.core.jcr;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;

import org.nuxeo.ecm.core.api.DocumentModel;
import org.nuxeo.ecm.core.api.DocumentModelFactory;
import org.nuxeo.ecm.core.api.DocumentModelList;
import org.nuxeo.ecm.core.api.impl.DocumentModelListImpl;
import org.nuxeo.ecm.core.model.Document;
import org.nuxeo.ecm.core.model.DocumentException;
import org.nuxeo.ecm.core.query.QueryException;
import org.nuxeo.ecm.core.query.QueryResult;

public class JCRQueryXPathResult implements QueryResult {

    javax.jcr.query.QueryResult jcrQueryResult;

    NodeIterator iterator;

    Node node;

    JCRQueryXPath query;

    public JCRQueryXPathResult(JCRQueryXPath query,
            javax.jcr.query.QueryResult qr) throws RepositoryException {
        jcrQueryResult = qr;
        iterator = qr.getNodes();
        this.query = query;
    }

    public long count() {
        return iterator.getSize();

    }

    public boolean isEmpty() {
        if (iterator.getPosition() > 0) {
            return false;
        }
        return iterator.hasNext();
    }

    public boolean next() {
        if (iterator.hasNext()) {
            node = iterator.nextNode();
            return true;
        }
        return false;
    }

    public long row() {
        return iterator.getPosition();
    }

    /*
     * Return the results as a List
     */
    public DocumentModelList getDocumentModels() throws QueryException {
        try {
            List<DocumentModel> list = new ArrayList<DocumentModel>();
            String[] schemas = null;
            while (next()) {
                if (node.getPath().equals("/ecm:root")) {
                    continue;
                }

                JCRDocument doc = new JCRDocument(query.session, node);
                if (schemas == null) {
                    schemas = getSchemasInSelectClause(doc);
                }
                DocumentModel docModel = DocumentModelFactory
                        .createDocumentModel(doc, schemas);
                list.add(docModel);
            }
            return new DocumentModelListImpl(list);
        } catch (Exception e) {
            e.printStackTrace();
            throw new QueryException("getDocumentModels failed", e);
        }
    }

    private String[] getSchemasInSelectClause(Document doc) {
        /*
         SelectList selectElements = query.sqlQuery.select.elements;
         // if it is an 'select * from', we return all document schemas
         if(selectElements.size()==0) {
         DocumentType docType = doc.getType();
         return docType.getSchemaNames();
         }
         */

        // otherwise, we check for each schema if it contains any field in the select clause
        List<String> schemaNames = new ArrayList<String>();

        // make sure the schema "common" is always selected
        if (!schemaNames.contains("common")) {
            schemaNames.add("common");
        }

        //return schemaNames.toArray(new String[schemaNames.size()]);

        // NXP-415: fields are empty sometimes - it is not desirable to load
        // all schemas, though
        // XXX : lazy loading should work better
        return null; // force to load all schemas for documents...
    }

    public Object getObject() throws QueryException {
        JCRDocument document;
        try {
            document = getOwnerDocument(node);
        } catch (RepositoryException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            return null;
        }

        // we have the document with the found node as on of its children
        if (true) {
            return document;
        }

        try {
            return document.exportMap((String[]) null);
        } catch (DocumentException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Creates a JCRDocument for the parent node of the given node.
     *
     * @param node
     * @return
     * @throws RepositoryException
     * @throws QueryException
     */
    private JCRDocument getOwnerDocument(Node node) throws RepositoryException,
            QueryException {
        //Node parent = ModelAdapter.getParentNode(node); - don't use this: will jump over direct parent
        final Node parent = node.getParent();
        if (parent == null) {
            final String nodeDesc = node.getPath();
            throw new QueryException("orphan content node detected: "
                    + nodeDesc);
        }

        final JCRDocument document = new JCRDocument(query.session, node);

        return document;
    }

    public boolean getBoolean(int i) throws QueryException {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean getBoolean(String column) throws QueryException {
        // TODO Auto-generated method stub
        return false;
    }

    public double getDouble(int i, double defaultValue) throws QueryException {
        // TODO Auto-generated method stub
        return 0;
    }

    public double getDouble(String column, double defaultValue)
            throws QueryException {
        // TODO Auto-generated method stub
        return 0;
    }

    public long getLong(int i, long defaultValue) throws QueryException {
        // TODO Auto-generated method stub
        return 0;
    }

    public long getLong(String column, long defaultValue) throws QueryException {
        // TODO Auto-generated method stub
        return 0;
    }

    public Object getObject(String column) throws QueryException {
        // TODO Auto-generated method stub
        return null;
    }

    public String getString(int i) throws QueryException {
        // TODO Auto-generated method stub
        return null;
    }

    public String getString(String column) throws QueryException {
        // TODO Auto-generated method stub
        return null;
    }

}
