package org.nuxeo.ecm.core.jcr.search.jrplug;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;

import javax.jcr.RepositoryException;

import org.apache.jackrabbit.core.NodeId;
import org.apache.jackrabbit.core.NodeIdIterator;
import org.apache.jackrabbit.core.query.lucene.NamespaceMappings;
import org.apache.jackrabbit.core.query.lucene.SearchIndex;
import org.apache.jackrabbit.core.state.NodeState;
import org.apache.jackrabbit.core.state.NodeStateIterator;
import org.apache.lucene.document.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AdvSearchIndex extends SearchIndex {

    /**
     * The logger instance for this class.
     */
    private static final Logger log = LoggerFactory
            .getLogger(AdvSearchIndex.class);

    protected void doInit() throws IOException {
        super.doInit();
        log.info("<doInit>");
    }

    List<NodeState> indexedNodes = new ArrayList<NodeState>();

    /**
     * Creates a lucene <code>Document</code> from a node state using the
     * namespace mappings <code>nsMappings</code>.
     * 
     * @param node
     *            the node state to index.
     * @param nsMappings
     *            the namespace mappings of the search index.
     * @return a lucene <code>Document</code> that contains all properties of
     *         <code>node</code>.
     * @throws RepositoryException
     *             if an error occurs while indexing the <code>node</code>.
     */
    @Override
    protected Document createDocument(NodeState node,
            NamespaceMappings nsMappings) throws RepositoryException {

        if (indexedNodes.contains(node)) {
            return null; // already indexed by this indexing "session"
        }
        indexedNodes.add(node);

        final List textFilters = super.getTextFilters();
        // return DocNodeIndexer.createDocument(node,
        // getContext().getItemStateManager(), nsMappings, textFilters);

        DocNodeIndexer indexer = new DocNodeIndexer(node, getContext()
                .getItemStateManager(), nsMappings, textFilters);
        Document doc = indexer.createDoc();

        if (doc == null) {
            // contentNode = indexer.contentNode;
            contentNodes.add(indexer.contentNode);
        }

        return doc;
    }

    List<NodeState> contentNodes = new ArrayList<NodeState>();

    @Override
    public void updateNodes(NodeIdIterator remove, NodeStateIterator add)
            throws RepositoryException, IOException {
        indexedNodes.clear();
        // TODO Auto-generated method stub
        super.updateNodes(remove, add);

        for (NodeState contentNode : contentNodes) {
            updateNode(contentNode);
        }
    }

    private void updateNode(NodeState nod) throws RepositoryException {

        NodeIdIterator remove = createNodeIdIterator(nod);
        NodeStateIterator add = createNodeStateIterator(nod);

        try {
            super.updateNodes(remove, add);
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

    }

    private NodeIdIterator createNodeIdIterator(final NodeState node) {
        NodeIdIterator removedIds = new NodeIdIterator() {
            // private final Iterator iter = removedNodes.iterator();
            int i = 1;

            public NodeId nextNodeId() throws NoSuchElementException {
                return (NodeId) node.getNodeId();
            }

            public void remove() {
                throw new UnsupportedOperationException();
            }

            public boolean hasNext() {
                return i-- > 0;
            }

            public Object next() {
                return nextNodeId();
            }
        };
        return removedIds;
    }

    private NodeStateIterator createNodeStateIterator(final NodeState node) {
        NodeStateIterator addedStates = new NodeStateIterator() {
            int i = 1;

            public void remove() {
                throw new UnsupportedOperationException();
            }

            public boolean hasNext() {
                return i-- > 0;
            }

            public Object next() {
                return nextNodeState();
            }

            public NodeState nextNodeState() {
                /*
                 * NodeState item = null; NodeId id= (NodeId) iter.next(); try {
                 * item = (NodeState) itemMgr.getItemState(id); } catch
                 * (ItemStateException e) { log.error("Unable to index node " +
                 * id + ": does not exist"); } return item;
                 */
                return node;
            }
        };
        return addedStates;
    }

}
